//-----------------------------------------------------------------------------
// File: texture_filter.cpp
//
// Desc: DirectX window application created by the DirectX AppWizard
//-----------------------------------------------------------------------------
#define STRICT
#include <windows.h>
#include <commctrl.h>
#include <commdlg.h>
#include <basetsd.h>
#include <math.h>
#include <stdio.h>
#include <d3dx9.h>
#include <dxerr9.h>
#include <tchar.h>
#include "DXUtil.h"
#include "D3DEnumeration.h"
#include "D3DSettings.h"
#include "D3DApp.h"
#include "D3DFont.h"
#include "D3DFile.h"
#include "D3DUtil.h"
#include "resource.h"
#include "texture_filter.h"



//-----------------------------------------------------------------------------
// Global access to the app (needed for the global WndProc())
//-----------------------------------------------------------------------------
CMyD3DApplication* g_pApp  = NULL;
HINSTANCE          g_hInst = NULL;




//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: Entry point to the program. Initializes everything, and goes into a
//       message-processing loop. Idle time is used to render the scene.
//-----------------------------------------------------------------------------
INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT )
{
    CMyD3DApplication d3dApp;

    g_pApp  = &d3dApp;
    g_hInst = hInst;

    InitCommonControls();
    if( FAILED( d3dApp.Create( hInst ) ) )
        return 0;

    return d3dApp.Run();
}




//-----------------------------------------------------------------------------
// Name: CMyD3DApplication()
// Desc: Application constructor.   Paired with ~CMyD3DApplication()
//       Member variables should be initialized to a known state here.  
//       The application window has not yet been created and no Direct3D device 
//       has been created, so any initialization that depends on a window or 
//       Direct3D should be deferred to a later stage. 
//-----------------------------------------------------------------------------
CMyD3DApplication::CMyD3DApplication()
{
    m_dwCreationWidth           = 500;
    m_dwCreationHeight          = 375;
    m_strWindowTitle            = TEXT( "texture_filter" );
    m_d3dEnumeration.AppUsesDepthBuffer   = TRUE;
	m_bStartFullscreen			= false;
	m_bShowCursorWhenFullscreen	= false;

    // Create a D3D font using d3dfont.cpp
    m_pFont                     = new CD3DFont( _T("Arial"), 12, D3DFONT_BOLD );
    m_bLoadingApp               = TRUE;

    ZeroMemory( &m_UserInput, sizeof(m_UserInput) );

	m_fViewRotYaxis					= D3DX_PI;//0.0f;
	m_fViewRotXaxis					= D3DX_PI*3/7; //0.0f;

	m_vecFromPt = D3DXVECTOR3( 0, 0, 0 );
	GetCursorPos( &m_ptPrevCursor );
	ScreenToClient( m_hWnd, &m_ptPrevCursor );

	/*
	m_TextureFilterType		= D3DTEXF_LINEAR;
	m_TextureMipFilterType		= D3DTEXF_LINEAR;
	m_MaxAnisotropy			=	16;
	m_dwTargetTextureStage	= 0;
	*/
	m_bTextureBlendAlpha	= FALSE;

	this->m_ppd3dTexturePipe = &m_pd3dTexture0;
	this->m_pd3dTexture0	= NULL;
	this->m_pd3dTexture1	= NULL;
	this->m_pd3dTextureByUser = NULL;
	this->m_pd3dxMeshPipe	= NULL;
	this->m_pd3dxMeshWall	= NULL;

	this->m_d3dSettings.SetPresentInterval( D3DPRESENT_INTERVAL_DEFAULT );
    // Read settings from registry
    ReadSettings();
}




//-----------------------------------------------------------------------------
// Name: ~CMyD3DApplication()
// Desc: Application destructor.  Paired with CMyD3DApplication()
//-----------------------------------------------------------------------------
CMyD3DApplication::~CMyD3DApplication()
{
}




//-----------------------------------------------------------------------------
// Name: OneTimeSceneInit()
// Desc: Paired with FinalCleanup().
//       The window has been created and the IDirect3D9 interface has been
//       created, but the device has not been created yet.  Here you can
//       perform application-related initialization and cleanup that does
//       not depend on a device.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::OneTimeSceneInit()
{
    // TODO: perform one time initialization

    // Drawing loading status message until app finishes loading
    SendMessage( m_hWnd, WM_PAINT, 0, 0 );

    m_bLoadingApp = FALSE;

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: ReadSettings()
// Desc: Read the app settings from the registry
//-----------------------------------------------------------------------------
VOID CMyD3DApplication::ReadSettings()
{
    HKEY hkey;
    if( ERROR_SUCCESS == RegCreateKeyEx( HKEY_CURRENT_USER, DXAPP_KEY, 
        0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hkey, NULL ) )
    {
        // TODO: change as needed

        // Read the stored window width/height.  This is just an example,
        // of how to use DXUtil_Read*() functions.
        DXUtil_ReadIntRegKey( hkey, TEXT("Width"), &m_dwCreationWidth, m_dwCreationWidth );
        DXUtil_ReadIntRegKey( hkey, TEXT("Height"), &m_dwCreationHeight, m_dwCreationHeight );

        RegCloseKey( hkey );
    }
}




//-----------------------------------------------------------------------------
// Name: WriteSettings()
// Desc: Write the app settings to the registry
//-----------------------------------------------------------------------------
VOID CMyD3DApplication::WriteSettings()
{
    HKEY hkey;

    if( ERROR_SUCCESS == RegCreateKeyEx( HKEY_CURRENT_USER, DXAPP_KEY, 
        0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hkey, NULL ) )
    {
        // TODO: change as needed

        // Write the window width/height.  This is just an example,
        // of how to use DXUtil_Write*() functions.
        DXUtil_WriteIntRegKey( hkey, TEXT("Width"), m_rcWindowClient.right );
        DXUtil_WriteIntRegKey( hkey, TEXT("Height"), m_rcWindowClient.bottom );

        RegCloseKey( hkey );
    }
}





//-----------------------------------------------------------------------------
// Name: ConfirmDevice()
// Desc: Called during device initialization, this code checks the display device
//       for some minimum set of capabilities
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::ConfirmDevice( D3DCAPS9* pCaps, DWORD dwBehavior,
                                          D3DFORMAT Format )
{
    UNREFERENCED_PARAMETER( Format );
    UNREFERENCED_PARAMETER( dwBehavior );
    UNREFERENCED_PARAMETER( pCaps );
    
    BOOL bCapsAcceptable;

    // TODO: Perform checks to see if these display caps are acceptable.
    bCapsAcceptable = TRUE;

    if( bCapsAcceptable )         
        return S_OK;
    else
        return E_FAIL;
}




//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: Paired with DeleteDeviceObjects()
//       The device has been created.  Resources that are not lost on
//       Reset() can be created here -- resources in D3DPOOL_MANAGED,
//       D3DPOOL_SCRATCH, or D3DPOOL_SYSTEMMEM.  Image surfaces created via
//       CreateOffScreenPlainSurface are never lost and can be created here.  Vertex
//       shaders and pixel shaders can also be created here as they are not
//       lost on Reset().
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::InitDeviceObjects()
{
	// Init Menu & related value.
	PostMessage(m_hWnd, WM_COMMAND, IDM_TEXTURESTAGE_1, 0 );
	PostMessage(m_hWnd, WM_COMMAND, IDM_TEXTURE_1, 0 );
	PostMessage(m_hWnd, WM_COMMAND, IDM_TEXTUREFILTER_LINEARMIPLINEAR, 0 );
	PostMessage(m_hWnd, WM_COMMAND, IDM_ANISOTROPY_16, 0 );
	PostMessage(m_hWnd, WM_COMMAND, IDM_TEXTUREBLENDALPHA, 0 );

    // TODO: create device objects
    HRESULT hr;

    // Init the font
    hr = m_pFont->InitDeviceObjects( m_pd3dDevice );
    if( FAILED( hr ) )
        return DXTRACE_ERR( "m_pFont->InitDeviceObjects", hr );

	if( FAILED( hr = D3DXLoadMeshFromX( 
									_T("model/pipe.x"),
									D3DXMESH_MANAGED,
									m_pd3dDevice,
									NULL,
									NULL,
									NULL,
									NULL,
									&m_pd3dxMeshPipe) ) ){
		return DXTRACE_ERR( "D3DXLoadMeshFromX", hr );
	}
	if( FAILED( hr = D3DXLoadMeshFromX( 
									_T("model/wall.x"),
									D3DXMESH_MANAGED,
									m_pd3dDevice,
									NULL,
									NULL,
									NULL,
									NULL,
									&m_pd3dxMeshWall) ) ){
		return DXTRACE_ERR( "D3DXLoadMeshFromX", hr );
	}
	if( FAILED( D3DXCreateTextureFromFile( 
									m_pd3dDevice,
									_T( MYTEXTURE0 ),
									&this->m_pd3dTexture0 ) ) ){
		return DXTRACE_ERR( "D3DXCreateTextureFromFile: MYTEXTURE0", hr );
	}
	if( FAILED( D3DXCreateTextureFromFile( 
									m_pd3dDevice,
									_T( MYTEXTURE1 ),
									&this->m_pd3dTexture1 ) ) ){
		return DXTRACE_ERR( "D3DXCreateTextureFromFile: MYTEXTURE0", hr );
	}

	return S_OK;
}

//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc: Paired with InvalidateDeviceObjects()
//       The device exists, but may have just been Reset().  Resources in
//       D3DPOOL_DEFAULT and any other device state that persists during
//       rendering should be set here.  Render states, matrices, textures,
//       etc., that don't change during rendering can be set once here to
//       avoid redundant state setting during Render() or FrameMove().
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::RestoreDeviceObjects()
{
    // TODO: setup render states

    // Setup a material
    D3DMATERIAL9 mtrl;
    D3DUtil_InitMaterial( mtrl, 1.0f, 0.0f, 0.0f );
    m_pd3dDevice->SetMaterial( &mtrl );

    // Set up the textures
    m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE );
    m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
    m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );
    m_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP,   D3DTOP_MODULATE );
    m_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE );
    m_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE );
    m_pd3dDevice->SetSamplerState( 0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR );
    m_pd3dDevice->SetSamplerState( 0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR );

    // Set miscellaneous render states
    m_pd3dDevice->SetRenderState( D3DRS_DITHERENABLE,   FALSE );
    m_pd3dDevice->SetRenderState( D3DRS_SPECULARENABLE, FALSE );
    m_pd3dDevice->SetRenderState( D3DRS_ZENABLE,        D3DZB_TRUE );
    m_pd3dDevice->SetRenderState( D3DRS_AMBIENT,        0x000F0F0F );

    // Set the world matrix
    /*
	D3DXMATRIX matIdentity;
    D3DXMatrixIdentity( &matIdentity );
    m_pd3dDevice->SetTransform( D3DTS_WORLD,  &matIdentity );
	*/

    // Set up our view matrix. A view matrix can be defined given an eye point,
    // a point to lookat, and a direction for which way is up. Here, we set the
    // eye five units back along the z-axis and up three units, look at the
    // origin, and define "up" to be in the y-direction.
	/*
    D3DXMATRIX matView;
    D3DXVECTOR3 vFromPt   = D3DXVECTOR3( 0.0f, 0.0f, -5.0f );
    D3DXVECTOR3 vLookatPt = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
    D3DXVECTOR3 vUpVec    = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );
    D3DXMatrixLookAtLH( &matView, &vFromPt, &vLookatPt, &vUpVec );
    m_pd3dDevice->SetTransform( D3DTS_VIEW, &matView );
	*/

    // Set the projection matrix
    D3DXMATRIX matProj;
    FLOAT fAspect = ((FLOAT)m_d3dsdBackBuffer.Width) / m_d3dsdBackBuffer.Height;
    //D3DXMatrixPerspectiveFovLH( &matProj, D3DX_PI/4, fAspect, 100.0f, 40000.0f ); // like RvS
	D3DXMatrixPerspectiveLH( &matProj, m_d3dsdBackBuffer.Width/10.0f, m_d3dsdBackBuffer.Height/10.0f, 100.0f, 40000.0f ); // like RS
    m_pd3dDevice->SetTransform( D3DTS_PROJECTION, &matProj );

    // Set up lighting states
	/*
    D3DLIGHT9 light;
    D3DUtil_InitLight( light, D3DLIGHT_DIRECTIONAL, -1.0f, -1.0f, 2.0f );
    m_pd3dDevice->SetLight( 0, &light );
    m_pd3dDevice->LightEnable( 0, TRUE );
    m_pd3dDevice->SetRenderState( D3DRS_LIGHTING, TRUE );
	*/

    // Restore the font
    m_pFont->RestoreDeviceObjects();

	//Update ( ClientArea == Buckbuffer Size )
	 IDirect3DSurface9 * pd3dSurface;
	 D3DSURFACE_DESC d3dSurfaceDesc;
	m_pd3dDevice->GetBackBuffer( 0, 0, D3DBACKBUFFER_TYPE_MONO, &pd3dSurface );
	pd3dSurface->GetDesc( &d3dSurfaceDesc );
	m_ptBackBuffer.x = d3dSurfaceDesc.Width;
	m_ptBackBuffer.y = d3dSurfaceDesc.Height;
	pd3dSurface->Release();

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::FrameMove()
{
    // TODO: update world

    // Update user input state
    UpdateInput( &m_UserInput );

	//evaluate View & Up directional vector.
	POINT ptCursor;
	FLOAT fLookMultiplyer = 6.0f;
	GetCursorPos( &ptCursor );
	if( m_UserInput.bLMouse || m_UserInput.bRMouse){ // Look around only when Mouse Button is pushed.
		m_fViewRotYaxis += fLookMultiplyer * (float) asin( ( ptCursor.x - m_ptPrevCursor.x ) / (float) m_ptBackBuffer.x );
		m_fViewRotXaxis += fLookMultiplyer * (float) asin( - ( ptCursor.y - m_ptPrevCursor.y ) / (float) m_ptBackBuffer.y );
	}
	if( ! m_bWindowed ){ //If fullscreen-ed, lock a MouseCursor at the center of the screen. 
		ptCursor.x = m_ptBackBuffer.x / 2;
		ptCursor.y = m_ptBackBuffer.y / 2;
		ClientToScreen( m_hWnd, &ptCursor );
		SetCursorPos( ptCursor.x, ptCursor.y );
	}
	m_ptPrevCursor = ptCursor;
	D3DXVECTOR3 vecViewDirXZ( (float) sin( m_fViewRotYaxis ), 0, (float) cos( m_fViewRotYaxis ) ); // Rotate around Y-axis
	D3DXVECTOR3	vecViewDirYZ( 0, (float) sin( m_fViewRotXaxis ), (float) cos( m_fViewRotXaxis ) ); // Rotate around X-axis
	D3DXVECTOR3 vecViewDirXYZ( (float) vecViewDirYZ.z * vecViewDirXZ.x , vecViewDirYZ.y, (float) vecViewDirYZ.z * vecViewDirXZ.z ); // ViewDirection vector
	D3DXVECTOR3	vecUpDirXYZ( - (float)( vecViewDirYZ.y * vecViewDirXZ.x ), vecViewDirYZ.z, - (float)( vecViewDirYZ.y * vecViewDirXZ.z ) ); //UpDirection vector

	// evaluate FromPt move.
	D3DXVECTOR3 vecMoveAmount( 0, 0, 0);
	FLOAT fMoveForward;
	FLOAT fMoveStrafe;
	FLOAT fMoveMultiplyer = 900.0f;

	if( m_UserInput.bRMouse ) //Turbo mode
		fMoveMultiplyer *= 3;

	{ // evaluate Strafe Movement.
		if( m_UserInput.bMoveStrafeLeft && !m_UserInput.bMoveStrafeRight ){
			fMoveStrafe = m_fElapsedTime * fMoveMultiplyer;
		}else if( m_UserInput.bMoveStrafeRight && !m_UserInput.bMoveStrafeLeft ){
			fMoveStrafe = - m_fElapsedTime * fMoveMultiplyer;
		}else{
			fMoveStrafe = 0;
		}
		if( fMoveStrafe != 0 ){
			vecMoveAmount += D3DXVECTOR3( -fMoveStrafe * vecViewDirXZ.z, 0, fMoveStrafe * vecViewDirXZ.x );
		}
	}
	{ // evaluate Forward Movement.
		if( m_UserInput.bMoveForward && !m_UserInput.bMoveBackward ){
			fMoveForward = m_fElapsedTime * fMoveMultiplyer;
		}else if( m_UserInput.bMoveBackward && !m_UserInput.bMoveForward ){
			fMoveForward = - m_fElapsedTime * fMoveMultiplyer;
		}else{
			fMoveForward = 0;
		}
		if(fMoveForward != 0 ){
			vecMoveAmount += fMoveForward * vecViewDirXYZ;
		}
	}

	//evaluate new vecFromPt.
	m_vecFromPt += vecMoveAmount; // No obstacles.

	// evaluate View matrix.
	D3DXMATRIXA16	matView;
	D3DXVECTOR3	vecLookatPt = m_vecFromPt + vecViewDirXYZ;
	D3DXMatrixLookAtLH( &matView, &m_vecFromPt, &vecLookatPt, &vecUpDirXYZ );
	m_pd3dDevice->SetTransform( D3DTS_VIEW, &matView );

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: UpdateInput()
// Desc: Update the user input.  Called once per frame 
//-----------------------------------------------------------------------------
void CMyD3DApplication::UpdateInput( UserInput* pUserInput )
{
    pUserInput->bRotateUp    = ( m_bActive && (GetAsyncKeyState( VK_UP )    & 0x8000) == 0x8000 );
    pUserInput->bRotateDown  = ( m_bActive && (GetAsyncKeyState( VK_DOWN )  & 0x8000) == 0x8000 );
    pUserInput->bRotateLeft  = ( m_bActive && (GetAsyncKeyState( VK_LEFT )  & 0x8000) == 0x8000 );
    pUserInput->bRotateRight = ( m_bActive && (GetAsyncKeyState( VK_RIGHT ) & 0x8000) == 0x8000 );

	pUserInput->bMoveForward = ( m_bActive && (GetAsyncKeyState( 'W' ) & 0x8000) == 0x8000 );
	pUserInput->bMoveBackward = ( m_bActive && (GetAsyncKeyState( 'S' ) & 0x8000) == 0x8000 );
	pUserInput->bMoveStrafeLeft = ( m_bActive && (GetAsyncKeyState( 'A' ) & 0x8000) == 0x8000 );
	pUserInput->bMoveStrafeRight = ( m_bActive && (GetAsyncKeyState( 'D' ) & 0x8000) == 0x8000 );

	pUserInput->bLMouse =  ( m_bActive && (GetAsyncKeyState(VK_LBUTTON  ) & 0x8000) == 0x8000 );
	pUserInput->bRMouse =  ( m_bActive && (GetAsyncKeyState( VK_RBUTTON ) & 0x8000) == 0x8000 );
}




//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::Render()
{
    // Clear the viewport
    m_pd3dDevice->Clear( 0L, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,
                         0x00000000, 1.0f, 0L );

	this->SetTextureFilterState();
    // Begin the scene
    if( SUCCEEDED( m_pd3dDevice->BeginScene() ) )
    {
        // TODO: render world
 
		D3DXMATRIXA16 mat;
		D3DXMATRIXA16 matScale;
		D3DXMATRIXA16 matRotate;
		D3DXMATRIXA16 matMove;
		FLOAT scaling;


		m_pd3dDevice->SetPixelShader( NULL );
		m_pd3dDevice->SetVertexShader( NULL );
		m_pd3dDevice->LightEnable( 0, FALSE );
		m_pd3dDevice->SetRenderState( D3DRS_TEXTUREFACTOR, 0x2f000000 ); //Use alpha value only.
		if( m_bTextureBlendAlpha ){
			DWORD i = 0;
			m_pd3dDevice->SetTexture( i, *m_ppd3dTexturePipe );
			m_pd3dDevice->SetTextureStageState( i, D3DTSS_COLOROP,   D3DTOP_SELECTARG1 );
			m_pd3dDevice->SetTextureStageState( i, D3DTSS_TEXCOORDINDEX,  0 );
			m_pd3dDevice->SetTextureStageState( i, D3DTSS_COLORARG1, D3DTA_TEXTURE );
			++i;
			for( ; i <= m_dwTargetTextureStage; ++i ){
				m_pd3dDevice->SetTexture( i, m_pd3dTexture0 );
				m_pd3dDevice->SetTextureStageState( i, D3DTSS_COLOROP,   D3DTOP_BLENDFACTORALPHA );
				m_pd3dDevice->SetTextureStageState( i, D3DTSS_TEXCOORDINDEX,  0 );
				m_pd3dDevice->SetTextureStageState( i, D3DTSS_COLORARG1, D3DTA_TEXTURE );
				m_pd3dDevice->SetTextureStageState( i, D3DTSS_COLORARG2, D3DTA_CURRENT );
			}
			for( ; i < 8; ++i ){
				m_pd3dDevice->SetTextureStageState( i, D3DTSS_COLOROP,  D3DTOP_DISABLE  );
			}
		}else{
			DWORD i = 0;
			for( ; i < m_dwTargetTextureStage; ++i ){
				m_pd3dDevice->SetTexture( i, NULL );
				m_pd3dDevice->SetTextureStageState( i, D3DTSS_COLOROP,   D3DTOP_SELECTARG1 );
				m_pd3dDevice->SetTextureStageState( i, D3DTSS_COLORARG1, D3DTA_DIFFUSE );
			}
			m_pd3dDevice->SetTexture( i, *m_ppd3dTexturePipe );
			m_pd3dDevice->SetTextureStageState( i, D3DTSS_COLOROP,   D3DTOP_SELECTARG1 );
			m_pd3dDevice->SetTextureStageState( i, D3DTSS_TEXCOORDINDEX,  0 );
			m_pd3dDevice->SetTextureStageState( i, D3DTSS_COLORARG1, D3DTA_TEXTURE );
			++i;
			for( ; i < 8; ++i ){
				m_pd3dDevice->SetTextureStageState( i, D3DTSS_COLOROP,  D3DTOP_DISABLE  );
			}
		}

		// Render Pipe mesh.
		scaling = 3.0f;
		const float z_scaling = .5f;
		D3DXMatrixScaling( &matScale, scaling, scaling, z_scaling*scaling );
		int i;
		for( i=1; i<50/z_scaling; ++i ){
			D3DXMatrixTranslation( &matMove, 0, 0, z_scaling*200*scaling*(i-0.5f) );
			D3DXMatrixMultiply( &mat, &matScale, &matMove );
			m_pd3dDevice->SetTransform( D3DTS_WORLD, &mat );
			m_pd3dxMeshPipe->DrawSubset( 0 );
			D3DXMatrixTranslation( &matMove, 0, 20000, 0 );
			D3DXMatrixMultiply( &mat, &mat, &matMove );
			m_pd3dDevice->SetTransform( D3DTS_WORLD, &mat );
			m_pd3dxMeshPipe->DrawSubset( 0 );
			D3DXMatrixTranslation( &matMove, 0, -40000, 0 );
			D3DXMatrixMultiply( &mat, &mat, &matMove );
			m_pd3dDevice->SetTransform( D3DTS_WORLD, &mat );
			m_pd3dxMeshPipe->DrawSubset( 0 );
		}

		// Render Wall Mesh
		scaling = 10.0f;
		for( i=-19; i<20; ++i ){
			D3DXMatrixTranslation( &matMove, 0, 100*scaling*(i), 0 );
			D3DXMatrixScaling( &matScale, scaling, scaling, scaling );
			D3DXMatrixMultiply( &mat, &matScale, &matMove );
			m_pd3dDevice->SetTransform( D3DTS_WORLD, &mat );
			m_pd3dxMeshWall->DrawSubset( 0 );

			if( i <= -19 ){
				D3DXMatrixMultiply( &mat, &matScale, D3DXMatrixRotationX( &matRotate, -D3DX_PI/2 ) );
				D3DXMatrixMultiply( &mat, &mat, &matMove );
				m_pd3dDevice->SetTransform( D3DTS_WORLD,  &mat );
				m_pd3dxMeshWall->DrawSubset( 0 );
			}else if( i >= 19 ){
				D3DXMatrixMultiply( &mat, &matScale, D3DXMatrixRotationX( &matRotate, D3DX_PI/2 ) );
				D3DXMatrixMultiply( &mat, &mat, &matMove );
				m_pd3dDevice->SetTransform( D3DTS_WORLD,  &mat );
				m_pd3dxMeshWall->DrawSubset( 0 );
			}
			if( i != 0 ){
				D3DXMatrixMultiply( &mat, &matScale, D3DXMatrixRotationY( &matRotate, D3DX_PI ) );
				D3DXMatrixMultiply( &mat, &mat, &matMove );
				m_pd3dDevice->SetTransform( D3DTS_WORLD,  &mat );
				m_pd3dxMeshWall->DrawSubset( 0 );
			}

			D3DXMatrixMultiply( &mat, &matScale, D3DXMatrixRotationY( &matRotate, D3DX_PI/2 ) );
			D3DXMatrixMultiply( &mat, &mat, &matMove );
			m_pd3dDevice->SetTransform( D3DTS_WORLD,  &mat );
			m_pd3dxMeshWall->DrawSubset( 0 );

			D3DXMatrixMultiply( &mat, &matScale, D3DXMatrixRotationY( &matRotate, -D3DX_PI/2 ) );
			D3DXMatrixMultiply( &mat, &mat, &matMove );
			m_pd3dDevice->SetTransform( D3DTS_WORLD,  &mat );
			m_pd3dxMeshWall->DrawSubset( 0 );
		}

        // Render stats and help text  
        RenderText();

        // End the scene.
        m_pd3dDevice->EndScene();
    }

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: RenderText()
// Desc: Renders stats and help text to the scene.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::RenderText()
{
    D3DCOLOR fontColor        = D3DCOLOR_ARGB(160,255,255,0);
    TCHAR szMsg[MAX_PATH] = TEXT("");
	TCHAR szResource[MAX_PATH] = _T("");

    // Output display stats
    FLOAT fNextLine = 40.0f; 

    lstrcpy( szMsg, m_strDeviceStats );
    fNextLine -= 20.0f;
    m_pFont->DrawText( 2, fNextLine, fontColor, szMsg );

    lstrcpy( szMsg, m_strFrameStats );
    fNextLine -= 20.0f;
    m_pFont->DrawText( 2, fNextLine, fontColor, szMsg );

    // Output statistics & help
    fNextLine = (FLOAT) m_d3dsdBackBuffer.Height; 

	/*
	LoadString( GetModuleHandle(NULL), IDS_INPUT_STATUS, szResource, MAX_PATH );
    _stprintf( szMsg, szResource,
              //m_UserInput.bRotateUp, m_UserInput.bRotateDown, m_UserInput.bRotateLeft, m_UserInput.bRotateRight,
			  m_UserInput.bLMouse,
			  m_UserInput.bMoveForward, m_UserInput.bMoveBackward, m_UserInput.bMoveStrafeLeft, m_UserInput.bMoveStrafeRight );
    fNextLine -= 20.0f; m_pFont->DrawText( 2, fNextLine, fontColor, szMsg );
	*/

	/*
	LoadString( GetModuleHandle(NULL), IDS_USAGE_LOOK, szResource, MAX_PATH );
    lstrcpy( szMsg, szResource );
    fNextLine -= 20.0f; m_pFont->DrawText( 2, fNextLine, fontColor, szMsg );

	LoadString( GetModuleHandle(NULL), IDS_USAGE_TURBO, szResource, MAX_PATH );
    lstrcpy( szMsg, szResource );
    fNextLine -= 20.0f; m_pFont->DrawText( 2, fNextLine, fontColor, szMsg );

	LoadString( GetModuleHandle(NULL), IDS_USAGE_MOVE, szResource, MAX_PATH );
    lstrcpy( szMsg, szResource );
    fNextLine -= 20.0f; m_pFont->DrawText( 2, fNextLine, fontColor, szMsg );

	LoadString( GetModuleHandle(NULL), IDS_USAGE_CONFIGDEVICE, szResource, MAX_PATH );
    lstrcpy( szMsg, szResource );
    fNextLine -= 20.0f; m_pFont->DrawText( 2, fNextLine, fontColor, szMsg );
	*/

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: Overrrides the main WndProc, so the sample can do custom message
//       handling (e.g. processing mouse, keyboard, or menu commands).
//-----------------------------------------------------------------------------
LRESULT CMyD3DApplication::MsgProc( HWND hWnd, UINT msg, WPARAM wParam,
                                    LPARAM lParam )
{
    switch( msg )
    {
        case WM_PAINT:
        {
            if( m_bLoadingApp )
            {
                // Draw on the window tell the user that the app is loading
                // TODO: change as needed
                HDC hDC = GetDC( hWnd );
                TCHAR strMsg[MAX_PATH];
                wsprintf( strMsg, TEXT("Loading... Please wait") );
                RECT rct;
                GetClientRect( hWnd, &rct );
                DrawText( hDC, strMsg, -1, &rct, DT_CENTER|DT_VCENTER|DT_SINGLELINE );
                ReleaseDC( hWnd, hDC );
            }
            break;
        }

		case WM_COMMAND:
		{
			HMENU hMenu = GetMenu( hWnd );

			switch( LOWORD(wParam) )
			{
			case IDM_TEXTUREBLENDALPHA:
				DWORD dwCheck;
				m_bTextureBlendAlpha = ! m_bTextureBlendAlpha;
				dwCheck = m_bTextureBlendAlpha ? MF_CHECKED : MF_UNCHECKED;
				CheckMenuItem( 
						hMenu,
						IDM_TEXTUREBLENDALPHA,
						MF_BYCOMMAND | dwCheck );
				break;
			case IDM_TEXTURESTAGE_0:
			case IDM_TEXTURESTAGE_1:
			case IDM_TEXTURESTAGE_2:
			case IDM_TEXTURESTAGE_3:
			case IDM_TEXTURESTAGE_4:
			case IDM_TEXTURESTAGE_5:
			case IDM_TEXTURESTAGE_6:
			case IDM_TEXTURESTAGE_7:
				m_dwTargetTextureStage = LOWORD(wParam) % 100;
				CheckMenuRadioItem( hMenu, IDM_TEXTURESTAGE_0, IDM_TEXTURESTAGE_7, 
						LOWORD(wParam), MF_BYCOMMAND );
				break;

			case IDM_ANISOTROPY_1:
			case IDM_ANISOTROPY_2:
			case IDM_ANISOTROPY_4:
			case IDM_ANISOTROPY_8:
			case IDM_ANISOTROPY_16:
				this->m_MaxAnisotropy = LOWORD(wParam) % 100;
				CheckMenuRadioItem( hMenu, IDM_ANISOTROPY_1, IDM_ANISOTROPY_16, 
						LOWORD(wParam), MF_BYCOMMAND );
				break;

			case IDM_TEXTURE_0:
				m_ppd3dTexturePipe = &this->m_pd3dTexture0;
				goto CASE_TEXTURE;
			case IDM_TEXTURE_1:
				m_ppd3dTexturePipe = &this->m_pd3dTexture1;
				goto CASE_TEXTURE;
			case IDM_TEXTURE_OPEN:
				IDirect3DTexture9	*newTexture;
				OPENFILENAME ofn;
				TCHAR lptstrFile[MAX_PATH];
				TCHAR lptstrFileTitle[MAX_PATH] ;
				{
					LPCTSTR filter = "All(*.*)\0*.*\0\0";
					ZeroMemory( &ofn, sizeof(ofn) );
					ZeroMemory( lptstrFile, sizeof(lptstrFile) );
					ZeroMemory( lptstrFileTitle, sizeof(lptstrFileTitle) );
					ofn.lStructSize = sizeof(ofn);
					ofn.hwndOwner = hWnd;
					ofn.nMaxFile = 1;
					ofn.lpstrFilter = filter;
					ofn.lpstrFile = lptstrFile;
					ofn.nMaxFile = MAX_PATH;
					ofn.lpstrFileTitle = lptstrFileTitle;
					ofn.nMaxFileTitle = MAX_PATH;
				}
				if( GetOpenFileName( &ofn ) ){
					if( SUCCEEDED( D3DXCreateTextureFromFileEx(
													m_pd3dDevice,
													_T( ofn.lpstrFile ),
													D3DX_DEFAULT, D3DX_DEFAULT, D3DX_DEFAULT, 0, 
													D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, 
													D3DX_DEFAULT, D3DX_DEFAULT,
													0, NULL, NULL,
													&newTexture ) ) ){
						SAFE_RELEASE( m_pd3dTextureByUser );
						m_pd3dTextureByUser = newTexture;
						m_ppd3dTexturePipe = &m_pd3dTextureByUser;

						TCHAR szMsg[MAX_PATH] = _T("");
						TCHAR szResource[MAX_PATH] = _T("");

						LoadString( GetModuleHandle(NULL), IDS_TEXTURE_OPEN, szResource, MAX_PATH );
						_stprintf( szMsg, szResource, lptstrFileTitle );
						ModifyMenu( 
								hMenu,
								IDM_TEXTURE_OPEN,
								MF_BYCOMMAND | MF_STRING,
								IDM_TEXTURE_OPEN,
								szMsg );
						
						goto CASE_TEXTURE;
					}

					TCHAR szMsg[MAX_PATH] = _T("");
					TCHAR szResource[MAX_PATH] = _T("");
					LoadString( GetModuleHandle(NULL), IDS_ERROR_CANNOTOPENTEXTURE, szResource, MAX_PATH );
					_stprintf( szMsg, szResource, lptstrFileTitle );
					MessageBox( NULL, szResource, 
						NULL, MB_ICONWARNING | MB_OK );
				}
				break;
CASE_TEXTURE:
				CheckMenuRadioItem( hMenu, IDM_TEXTURE_0, IDM_TEXTURE_OPEN, 
						LOWORD(wParam), MF_BYCOMMAND );
				break;


			case IDM_TEXTUREFILTER_NEAR:
				m_TextureFilterType = D3DTEXF_POINT;
				m_TextureMipFilterType = D3DTEXF_NONE;
				goto CASE_TEXTUREFILTER;
			case IDM_TEXTUREFILTER_NEARMIPNEAR:
				m_TextureFilterType = D3DTEXF_POINT;
				m_TextureMipFilterType = D3DTEXF_POINT;
				goto CASE_TEXTUREFILTER;
			case IDM_TEXTUREFILTER_NEARMIPLINEAR:
				m_TextureFilterType = D3DTEXF_POINT;
				m_TextureMipFilterType = D3DTEXF_LINEAR;
				goto CASE_TEXTUREFILTER;
			case IDM_TEXTUREFILTER_LINEAR:
				m_TextureFilterType = D3DTEXF_LINEAR ;
				m_TextureMipFilterType = D3DTEXF_NONE;
				goto CASE_TEXTUREFILTER;
			case IDM_TEXTUREFILTER_LINEARMIPNEAR:
				m_TextureFilterType = D3DTEXF_LINEAR ;
				m_TextureMipFilterType = D3DTEXF_POINT;
				goto CASE_TEXTUREFILTER;
			case IDM_TEXTUREFILTER_LINEARMIPLINEAR:
				m_TextureFilterType = D3DTEXF_LINEAR ;
				m_TextureMipFilterType = D3DTEXF_LINEAR ;
				goto CASE_TEXTUREFILTER;
			case IDM_TEXTUREFILTER_ANISOTROPIC_0:
				m_TextureFilterType = D3DTEXF_ANISOTROPIC;
				m_TextureMipFilterType = D3DTEXF_NONE;
				goto CASE_TEXTUREFILTER;
			case IDM_TEXTUREFILTER_ANISOTROPIC_1:
				m_TextureFilterType = D3DTEXF_ANISOTROPIC ;
				m_TextureMipFilterType = D3DTEXF_POINT;
				goto CASE_TEXTUREFILTER;
			case IDM_TEXTUREFILTER_ANISOTROPIC_2:
				m_TextureFilterType = D3DTEXF_ANISOTROPIC ;
				m_TextureMipFilterType = D3DTEXF_LINEAR ;
				goto CASE_TEXTUREFILTER;
CASE_TEXTUREFILTER:
				CheckMenuRadioItem( hMenu, IDM_TEXTUREFILTER_NEAR, IDM_TEXTUREFILTER_ANISOTROPIC_2, 
						LOWORD(wParam), MF_BYCOMMAND );
				break;
			default:
				break;
			}
		}

    }

    return CD3DApplication::MsgProc( hWnd, msg, wParam, lParam );
}




//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc: Invalidates device objects.  Paired with RestoreDeviceObjects()
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::InvalidateDeviceObjects()
{
    // TODO: Cleanup any objects created in RestoreDeviceObjects()
    m_pFont->InvalidateDeviceObjects();

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Paired with InitDeviceObjects()
//       Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.  
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::DeleteDeviceObjects()
{
    // TODO: Cleanup any objects created in InitDeviceObjects()
    m_pFont->DeleteDeviceObjects();

	SAFE_RELEASE( this->m_pd3dTexture0 );
	SAFE_RELEASE( this->m_pd3dTexture1 );
	SAFE_RELEASE( this->m_pd3dTextureByUser );
	SAFE_RELEASE( this->m_pd3dxMeshPipe );
	SAFE_RELEASE( this->m_pd3dxMeshWall );

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: FinalCleanup()
// Desc: Paired with OneTimeSceneInit()
//       Called before the app exits, this function gives the app the chance
//       to cleanup after itself.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::FinalCleanup()
{
    // TODO: Perform any final cleanup needed
    // Cleanup D3D font
    SAFE_DELETE( m_pFont );

    // Write the settings to the registry
    WriteSettings();

    return S_OK;
}


//------------------------------------------
// Name: SetTextureFilterState()
// Desc: Setup TextureFilterState
//------------------------------------------
void CMyD3DApplication::SetTextureFilterState()
{
	m_pd3dDevice->SetSamplerState( 0, D3DSAMP_MINFILTER, m_TextureFilterType );
	m_pd3dDevice->SetSamplerState( 0, D3DSAMP_MAGFILTER, m_TextureFilterType );
	m_pd3dDevice->SetSamplerState( 0, D3DSAMP_MIPFILTER, m_TextureMipFilterType );
	m_pd3dDevice->SetSamplerState( 0, D3DSAMP_MAXANISOTROPY, m_MaxAnisotropy );
	m_pd3dDevice->SetSamplerState( 1, D3DSAMP_MINFILTER, m_TextureFilterType );
	m_pd3dDevice->SetSamplerState( 1, D3DSAMP_MAGFILTER, m_TextureFilterType );
	m_pd3dDevice->SetSamplerState( 1, D3DSAMP_MIPFILTER, m_TextureMipFilterType );
	m_pd3dDevice->SetSamplerState( 1, D3DSAMP_MAXANISOTROPY, m_MaxAnisotropy );
	m_pd3dDevice->SetSamplerState( 2, D3DSAMP_MINFILTER, m_TextureFilterType );
	m_pd3dDevice->SetSamplerState( 2, D3DSAMP_MAGFILTER, m_TextureFilterType );
	m_pd3dDevice->SetSamplerState( 2, D3DSAMP_MIPFILTER, m_TextureMipFilterType );
	m_pd3dDevice->SetSamplerState( 2, D3DSAMP_MAXANISOTROPY, m_MaxAnisotropy );
	m_pd3dDevice->SetSamplerState( 3, D3DSAMP_MINFILTER, m_TextureFilterType );
	m_pd3dDevice->SetSamplerState( 3, D3DSAMP_MAGFILTER, m_TextureFilterType );
	m_pd3dDevice->SetSamplerState( 3, D3DSAMP_MIPFILTER, m_TextureMipFilterType );
	m_pd3dDevice->SetSamplerState( 3, D3DSAMP_MAXANISOTROPY, m_MaxAnisotropy );
	m_pd3dDevice->SetSamplerState( 4, D3DSAMP_MINFILTER, m_TextureFilterType );
	m_pd3dDevice->SetSamplerState( 4, D3DSAMP_MAGFILTER, m_TextureFilterType );
	m_pd3dDevice->SetSamplerState( 4, D3DSAMP_MIPFILTER, m_TextureMipFilterType );
	m_pd3dDevice->SetSamplerState( 4, D3DSAMP_MAXANISOTROPY, m_MaxAnisotropy );
	m_pd3dDevice->SetSamplerState( 5, D3DSAMP_MINFILTER, m_TextureFilterType );
	m_pd3dDevice->SetSamplerState( 5, D3DSAMP_MAGFILTER, m_TextureFilterType );
	m_pd3dDevice->SetSamplerState( 5, D3DSAMP_MIPFILTER, m_TextureMipFilterType );
	m_pd3dDevice->SetSamplerState( 5, D3DSAMP_MAXANISOTROPY, m_MaxAnisotropy );
	m_pd3dDevice->SetSamplerState( 6, D3DSAMP_MINFILTER, m_TextureFilterType );
	m_pd3dDevice->SetSamplerState( 6, D3DSAMP_MAGFILTER, m_TextureFilterType );
	m_pd3dDevice->SetSamplerState( 6, D3DSAMP_MIPFILTER, m_TextureMipFilterType );
	m_pd3dDevice->SetSamplerState( 6, D3DSAMP_MAXANISOTROPY, m_MaxAnisotropy );
	m_pd3dDevice->SetSamplerState( 7, D3DSAMP_MINFILTER, m_TextureFilterType );
	m_pd3dDevice->SetSamplerState( 7, D3DSAMP_MAGFILTER, m_TextureFilterType );
	m_pd3dDevice->SetSamplerState( 7, D3DSAMP_MIPFILTER, m_TextureMipFilterType );
	m_pd3dDevice->SetSamplerState( 7, D3DSAMP_MAXANISOTROPY, m_MaxAnisotropy );
}
